# noqa
DANGEROUS_PATTERNS = [
    r"ignore\s+(all\s+)?previous\s+instructions?",
    r"you\s+are\s+now\s+(in\s+)?developer\s+mode",
    r"system\s+override",
    r"reveal\s+prompt",
    r"pretend\s+to\s+be\s+",
    r"act\s+like\s+",
    r"forget\s+everything",
    r"###\s*system",
]
